function [fMAT,fvec,alphavec]=jacknifeA(sval,Dvec,Sigvec,RMAT,CMAT);
%
% Usage: [fMAT,fvec,alphavec]=jacknifeA(sval,Dvec,Sigvec,RMAT,CMAT);
%
% Jacknife Skilling MEMSYS5 algorithm for constructing an h distribition
% that maximizes the Bayesian probability of the data.
%
% Inputs: 	Dvec		data 
%			Sigvec		sig_data
%			RMAT		basis functions matrix
%			CMAT		intrinsic corellation function matrix
%
% Outputs:	fMAT		matrix of fvec solutions
%			fvec		optimal fvec from jacknife analysis
%			alphavec 	stop temperatures over jacknife
%
NITER=10;
Ndim=length(Dvec);[Mdim Ldim]=size(CMAT);
hvec=zeros(Ldim,1); fvec=zeros(Mdim,1); sfvec=zeros(Mdim,1);
fMAT=zeros(Mdim,NITER); alphavec=zeros(NITER,1);
% FIO=fopen('jacknife.txt','wt');
%
for iteration=1:NITER   
    [hvec,fvec,sfvec,cscas,alpha]=fms5A(sval,Dvec,Sigvec,RMAT,CMAT,0.0,0.05,0,1);
	fMAT(:,iteration)=fvec; alphavec(iteration)=alpha;
end
alpha_stop=mean(alphavec);
[hvec,fvec,sfvec,cscas,alpha]=fms5A(sval,Dvec,Sigvec,RMAT,CMAT,alpha_stop,0.05,1,0);
% fclose(FIO);
